import {expect} from 'vitest'
import { domTest } from './helpers.mjs'
import '../src-js/godo.js'

domTest('content is moved to linked input when linked', ({appendToDom}) => {
  // invalid-tag to check initial content is parsed before being assigned to
  // the input value
  const dom = appendToDom(`
    <div>
      <input type="hidden" id="marshmallow"></input>
      <input type="hidden" id="carambar"></input>
      <godo-editor linked-input="marshmallow">
        <invalid-tag>Initial</invalid-tag>
        <p>Content</p>
      </godo-editor>
    </div>
  `)
  const godo = dom.querySelector('godo-editor')
  const marshmallow = dom.querySelector('#marshmallow')
  const carambar = dom.querySelector('#carambar')

  expect(marshmallow.value).toBe('<p>Initial</p><p>Content</p>')
  expect(carambar.value).toBe('')
  // should've been removed
  expect(dom.querySelector('invalid-tag')).toBe(null)

  godo.setAttribute('linked-input', 'carambar')
  expect(marshmallow.value).toBe('<p>Initial</p><p>Content</p>')
  expect(carambar.value).toBe('<p>Initial</p><p>Content</p>')
})

domTest('godo content is updated from linked input on update-event', ({appendToDom}) => {
  // invalid-tag to check initial content is parsed before being assigned to
  // the text area value
  const dom = appendToDom(`
    <div>
      <input type="hidden" id="marshmallow"></input>
      <input type="hidden" id="carambar"></input>
      <godo-editor update-event="update-me" linked-input="marshmallow" >
      </godo-editor>
    </div>
  `)

  const godo = dom.querySelector('godo-editor')
  const godoContent = dom.querySelector('godo-editor .godo--editor')
  const marshmallow = dom.querySelector('#marshmallow')
  const carambar = dom.querySelector('#carambar')

  marshmallow.value = '<invalid-tag>Content changed</invalid-tag>'
  marshmallow.dispatchEvent(new Event('update-me'))
  expect(godoContent.innerHTML).toBe('<p>Content changed</p>')

  godo.setAttribute('linked-input', 'carambar')
  marshmallow.value = '<invalid-tag>Should be ignored</invalid-tag>'
  expect(godoContent.innerHTML).toBe('<p>Content changed</p>')

  carambar.value = '<invalid-tag>New linked input content</invalid-tag>'
  carambar.dispatchEvent(new Event('update-me'))
  expect(godoContent.innerHTML).toBe('<p>New linked input content</p>')
})

domTest('linked input value is updated on form submit', ({appendToDom}) => {
  const dom = appendToDom(`
    <div>
      <form>
        <input type="hidden" id="marshmallow"></input>
      </form>
      <form>
        <input type="hidden" id="carambar"></input>
      </form>
      <godo-editor update-event="update-me" linked-input="marshmallow" >
      </godo-editor>
    </div>
  `)

  const godo = dom.querySelector('godo-editor')
  const marshmallow = dom.querySelector('#marshmallow')
  const carambar = dom.querySelector('#carambar')

  godo.view.updateState(godo._createState('<p>Changed content</p>'))
  expect(marshmallow.value).toBe('')

  marshmallow.form.dispatchEvent(new Event('submit'))
  expect(marshmallow.value).toBe('<p>Changed content</p>')

  godo.view.updateState(godo._createState(''))
  godo.setAttribute('linked-input', 'carambar')
  godo.view.updateState(godo._createState('<p>Changed content</p>'))
  expect(carambar.value).toBe('')

  carambar.form.dispatchEvent(new Event('submit'))
  expect(carambar.value).toBe('<p>Changed content</p>')
})
